<?php
/**
 * Plugin PDF_VERSION pour Spip 3.x
 * Licence GPL (c) 2016 Cedric
 *
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

/**
 * Appel de wkhtml, soit par ligne de commande, soit par API HTTP sur un site qui en dispose en ligne de commande
 * @param $html_file
 * @param $pdf_file
 * @param array $args
 */
function inc_exec_wkhtmltopdf_dist($html_file, $pdf_file, $args = array()) {

	include_spip('inc/config');

	$methode = lire_config('pdf_version/methode','exec');
	if ($methode === 'exec'
	  and $wkhtmltopdf_path = lire_config('pdf_version/wkhtmltopdf_path','/usr/local/bin/wkhtmltopdf')) {

		// Appel de wkhtmltopdf en ligne de commande
		// on peut avoir plusieurs exec possibles

		$wkhtmltopdf_path = trim($wkhtmltopdf_path);
		$wkhtmltopdf_path_multiples = explode("\n", $wkhtmltopdf_path);
		$wkhtmltopdf_path_multiples = array_map('trim', $wkhtmltopdf_path_multiples);
		$wkhtmltopdf_path_multiples = array_filter($wkhtmltopdf_path_multiples);

		$args[] = $html_file;
		$args[] = $pdf_file;

		$cmd = "";
		foreach ($args as $k=>$v){
			if (!is_numeric($k)){
				$cmd .= " $k";
			}
			$cmd .= ' '.escapeshellarg($v);
		}

		$cmd .= '  2>&1';

		foreach ($wkhtmltopdf_path_multiples as $count => $wkhtmltopdf_path) {
			$cmd_exec = $wkhtmltopdf_path . $cmd;
			$prologue = ($count == 0 ? "" : "Essai ".($count+1).": ");
			spip_log($prologue . $cmd_exec,"wkhtmltopdf");

			$output = array();
			$return_var = 0;
			exec($cmd_exec, $output, $return_var);
			spip_log(implode("\n",$output),"wkhtmltopdf");

			if (!$return_var and file_exists($pdf_file) and filesize($pdf_file)) {
				return true;
			}

			spip_log("Erreur $return_var","wkhtmltopdf" . _LOG_ERREUR);

		}

		// si on arrive la c'est un echec apres avoir essaye toutes les variantes
		return false;
	}

	if($methode === 'http'
	  and $wkhtmltopdf_api_url = lire_config('pdf_version/wkhtmltopdf_api_url')) {

		// Appel de wkhtmltopdf via API http

		// lire le html et le renseigner dans les arguments
		$html = "";
		lire_fichier($html_file, $html);
		$args['html'] = normalise_retours_ligne_html($html);
		if (isset($args['--header-html'])) {
			lire_fichier($args['--header-html'], $args['--header-html']);
			$args['--header-html'] = normalise_retours_ligne_html($args['--header-html']);
		}
		if (isset($args['--footer-html'])) {
			lire_fichier($args['--footer-html'], $args['--footer-html']);
			$args['--footer-html'] = normalise_retours_ligne_html($args['--footer-html']);
		}

		$result = true;
		include_spip('inc/distant');
		if(function_exists('recuperer_url')) {
			$res = recuperer_url($wkhtmltopdf_api_url, array(
				'methode' => 'POST',
				'datas' => $args,
				'file' => "{$pdf_file}.tmp",
			));
			if (!$res or $res['status']>300) {
				@unlink("{$pdf_file}.tmp");
				$result = false;
			}
		}
		else {
			$res = recuperer_page($wkhtmltopdf_api_url, "{$pdf_file}.tmp", false, null, $args);
		}

		if ($result){
			$extract = file_get_contents("{$pdf_file}.tmp", false, null, 0, 16000);
			$extract = ltrim($extract);
			if (stripos($extract, '<!DOCTYPE html>')===0
				and stripos($extract, "<html")!==false
				and stripos($extract, "minipres")!==false){
				// c'est visiblement une minipres html, donc le serveur a foire et on a pas reconnus le status code :(
				@unlink("{$pdf_file}.tmp");
				$result = false;

				// renseigner le status pour les logs au lieu du 200
				if (isset($res['status'])){
					if (preg_match(",<title>(.*)</title>,Uims", $extract, $m)){
						$res['status'] .= " / " . $m[1];
					}
				}
			}
		}
		if ($result) {
			@rename("{$pdf_file}.tmp", $pdf_file);
		}
		else {
			@unlink("{$pdf_file}.tmp");
		}

		spip_log($wkhtmltopdf_api_url . " $html_file -> $pdf_file : " . var_export($res, true), "wkhtmltopdf". ($result ? '' : _LOG_ERREUR));

		return $result;
	}

	spip_log("methode $methode inconnue ou non configuree","wkhtmltopdf");
	return false;

}

function &normalise_retours_ligne_html(&$texte) {
	$texte = str_replace("\r\n", "\n", $texte);
	$texte = str_replace("\r", "\n", $texte);
	$texte = str_replace("\n", "\r\n", $texte);

	return $texte;
}
